from event_procedures.models import Event, Procedure
from django.core.exceptions import MiddlewareNotUsed
from django.dispatch import Signal
from settings import SIGNAL_MODULES
import importlib


class EventRegistration(object):
    # Middleware used to force event population, this can probably be
    # replaced in versions 1.7+ by the start signal

    def __init__(self):
        signals = {}
        for module_str in SIGNAL_MODULES:
            module = importlib.import_module(module_str).signals
            sigs = {module_str + '.signals.' + name: getattr(module, name) for name in dir(module) if isinstance(getattr(module, name), Signal)}
            signals.update(sigs)

        names = [name for name, signal in signals.items()]
        Event.objects.exclude(name__in=names).delete()  # get rid of events we don't want
        event_names = [event.name for event in Event.objects.all()]
        new_names = [name for name in names if name not in event_names]  # add events that we want, but aren't tracking
        for name in new_names:
            Event.objects.create(name=name)

        for procedure in Procedure.objects.all():
            procedure.connect()

        raise MiddlewareNotUsed



def extract_context_var(var, kwargs):
    extract_var = kwargs
    for key in var.split('.'):
        try:
            if hasattr(extract_var, '__getitem__'):
                extract_var = extract_var[key]
            else:
                extract_var = getattr(extract_var, key)
        except LookupError:
            raise LookupError(key + ' could not be found in ' + str(extract_var)
                              + '. Source address: ' + var)



# -*- coding: utf-8 -*-
from south.db import db
from south.v2 import SchemaMigration


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Event'
        db.create_table(u'event_procedures_event', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=50)),
        ))
        db.send_create_signal(u'event_procedures', ['Event'])

        # Adding model 'Action'
        db.create_table(u'event_procedures_action', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
        ))
        db.send_create_signal(u'event_procedures', ['Action'])

        # Adding model 'Procedure'
        db.create_table(u'event_procedures_procedure', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('event', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['event_procedures.Event'], null=True)),
            ('root', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['event_procedures.Action'], null=True)),
        ))
        db.send_create_signal(u'event_procedures', ['Procedure'])


    def backwards(self, orm):
        # Deleting model 'Event'
        db.delete_table(u'event_procedures_event')

        # Deleting model 'Action'
        db.delete_table(u'event_procedures_action')

        # Deleting model 'Procedure'
        db.delete_table(u'event_procedures_procedure')

    models = {
        u'event_procedures.action': {
            'Meta': {'object_name': 'Action'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        u'event_procedures.event': {
            'Meta': {'object_name': 'Event'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'event_procedures.procedure': {
            'Meta': {'object_name': 'Procedure'},
            'event': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['event_procedures.Event']", 'null': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'root': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['event_procedures.Action']", 'null': 'True'})
        },
    }

    complete_apps = ['event_procedures']



# -*- coding: utf-8 -*-
from south.db import db
from south.v2 import SchemaMigration


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'Action.parent'
        db.add_column(u'event_procedures_action', 'parent',
                      self.gf('django.db.models.fields.related.ForeignKey')(to=orm['event_procedures.Action'], null=True),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'Action.parent'
        db.delete_column(u'event_procedures_action', 'parent_id')


    models = {
        u'event_procedures.action': {
            'Meta': {'object_name': 'Action'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['event_procedures.Action']", 'null': 'True'})
        },
        u'event_procedures.event': {
            'Meta': {'object_name': 'Event'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'event_procedures.procedure': {
            'Meta': {'object_name': 'Procedure'},
            'event': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['event_procedures.Event']", 'null': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'root': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['event_procedures.Action']", 'null': 'True'})
        },
    }

    complete_apps = ['event_procedures']



from genericadmin.admin import GenericAdminModelAdmin, GenericTabularInline
from django.contrib import admin

from event_procedures.models import Action, Closure, Email, Procedure


class ProcedureAdmin(admin.ModelAdmin):
    model = Procedure


class ClosureInline(GenericTabularInline):
    model = Closure
    ct_field = "content_type"
    ct_fk_field = "object_id"


class ClosureAdmin(GenericAdminModelAdmin):
    model = Closure
    content_type_whitelist = ('event_procedures/email')


class ActionAdmin(GenericAdminModelAdmin):
    model = Action
    content_type_whitelist = ('event_procedures/email')


class EmailAdmin(admin.ModelAdmin):
    model = Email

admin.site.register(Action, ActionAdmin)
admin.site.register(Closure, ClosureAdmin)
admin.site.register(Email, EmailAdmin)
admin.site.register(Procedure, ProcedureAdmin)



# -*- coding: utf-8 -*-
from south.db import db
from south.v2 import SchemaMigration


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'Email.message_body'
        db.alter_column(u'event_procedures_email', 'message_body', self.gf('django.db.models.fields.CharField')(default='sample.html', max_length=256))

        # Changing field 'Email.message_html'
        db.alter_column(u'event_procedures_email', 'message_html', self.gf('django.db.models.fields.CharField')(default='sample.html', max_length=256))

    def backwards(self, orm):

        # Changing field 'Email.message_body'
        db.alter_column(u'event_procedures_email', 'message_body', self.gf('django.db.models.fields.CharField')(max_length=256, null=True))

        # Changing field 'Email.message_html'
        db.alter_column(u'event_procedures_email', 'message_html', self.gf('django.db.models.fields.CharField')(max_length=256, null=True))

    models = {
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'event_procedures.action': {
            'Meta': {'object_name': 'Action'},
            'closure': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['event_procedures.Closure']", 'null': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'default': "'An unnamed action'", 'max_length': '100'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['event_procedures.Action']", 'null': 'True', 'blank': 'True'})
        },
        u'event_procedures.closure': {
            'Meta': {'object_name': 'Closure'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']", 'null': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'default': "'An unnamed closure'", 'max_length': '100'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True'})
        },
        u'event_procedures.email': {
            'Meta': {'object_name': 'Email'},
            'bcc': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'cc': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'message_body': ('django.db.models.fields.CharField', [], {'max_length': '256'}),
            'message_html': ('django.db.models.fields.CharField', [], {'max_length': '256'}),
            'subject': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True'}),
            'to': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True'})
        },
        u'event_procedures.event': {
            'Meta': {'object_name': 'Event'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'event_procedures.procedure': {
            'Meta': {'object_name': 'Procedure'},
            'event': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['event_procedures.Event']", 'null': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'root': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['event_procedures.Action']", 'null': 'True'})
        },
    }

    complete_apps = ['event_procedures']



from django.conf import settings
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.core.mail import EmailMessage
from django.core.validators import validate_email
from django.db import models
from django.template import Template, Context

from event_procedures.utils import extract_context_var

import importlib
import logging
import os


class Event(models.Model):
    name = models.CharField(max_length=50)

    def delete(self, *args, **kwargs):
        logger = logging.getLogger(__name__)
        logger.info("Deleting event: " + self.name)
        logger.info("and associated procedures:")
        for procedure in Procedure.objects.filter(event=self):
            procedure.delete()
        super(Event, self).delete(*args, **kwargs)

    def __unicode__(self):
        return self.name


class Procedure(models.Model):
    event = models.ForeignKey('event_procedures.Event', null=True)
    root = models.ForeignKey('event_procedures.Action', null=True)

    @property
    def signal(self):
        module_name, _, class_name = self.event.name.rsplit('.', 2)
        return getattr(importlib.import_module(module_name).signals, class_name)

    def connect(self):
        logger = logging.getLogger(__name__)
        logger.info("Connecting " + str(self.root) + " to " + str(self.event))
        self.signal.connect(self.call, weak=False, dispatch_uid=str(self.id))

    def disconnect(self):
        logger = logging.getLogger(__name__)
        logger.info("Disconnecting " + str(self.root) + " from " + str(self.event))
        self.signal.disconnect(dispatch_uid=str(self.id))

    def save(self, *args, **kwargs):
        # When saved from django admin, the instance in memory
        # will have the new event loaded into memory, so we
        # must look up the previous version of the procedure
        # to disconnect it.
        try:
            Procedure.objects.get(id=self.id).disconnect()
        except Procedure.DoesNotExist:
            print "New procedure being created"
        super(Procedure, self).save(*args, **kwargs)
        self.connect()

    def delete(self, *args, **kwargs):
        logger = logging.getLogger(__name__)
        logger.info("Deleting procedure with id " + str(self.id) + "\
                     \nand associated with event " + str(self.event) + "\
                     \nand action " + str(self.root))
        self.disconnect()
        super(Procedure, self).delete(*args, **kwargs)

    def call(self, sender, **kwargs):
        kwargs.update({'sender': sender})
        self.root.call(**kwargs)

    def __unicode__(self):
        return "{} - {}".format(self.event, self.root)


class Action(models.Model):
    name = models.CharField(max_length=100, default="An unnamed action")
    parent = models.ForeignKey('event_procedures.Action', null=True, blank=True)

    closure = models.ForeignKey('event_procedures.Closure', null=True)

    @property
    def children(self):
        return list(Action.objects.filter(parent=self))

    def call(self, **kwargs):
        self.closure.call(**kwargs)
        for child in self.children:
            child.call(**kwargs)

    def __unicode__(self):
        return self.name


class Closure(models.Model):
    name = models.CharField(max_length=100, default="An unnamed closure")

    content_type = models.ForeignKey(ContentType, null=True)
    object_id = models.PositiveIntegerField(null=True)
    fn = generic.GenericForeignKey('content_type', 'object_id')

    def call(self, **kwargs):
        self.fn.call(**kwargs)

    def __unicode__(self):
        return self.name


class Email(models.Model):

    file_path = os.path.realpath(os.path.dirname(__file__)) + '/templates/'
    specs = [(path, files) for path, dirs, files in os.walk(file_path + 'email/') if not dirs]

    body_path, body_files = specs[0]
    body_file_paths = [body_path + '/' + file_name for file_name in body_files]
    BODY_CHOICES = zip(body_file_paths, body_files)

    html_path, html_files = specs[1]
    html_file_paths = [html_path + '/' + file_name for file_name in html_files]
    HTML_CHOICES = zip(html_file_paths, html_files)

    subject = models.CharField(max_length=100, null=True)
    message_html = models.CharField(max_length=256, choices=HTML_CHOICES)
    message_body = models.CharField(max_length=256, choices=BODY_CHOICES)
    to = models.CharField(max_length=50, null=True)
    cc = models.CharField(max_length=50, null=True, blank=True)
    bcc = models.CharField(max_length=50, null=True, blank=True)

    def prep_addresses(self, addresses, **kwargs):
        res = []
        addresses = [address for address in addresses if bool(address)]
        for address in addresses:
            address = address.strip()
            try:
                validate_email(address)
                res.append(address)
            except ValidationError:
                # invalid email, let's see if it is a deref from a
                # context variable
                extract_email = extract_context_var(address, kwargs)
                res.append(extract_email)
        return res

    def call(self, **kwargs):
        try:
            cc = self.prep_addresses(self.cc.split(';'), **kwargs) if self.cc is not None else None
            bcc = self.prep_addresses(self.bcc.split(';'), **kwargs) if self.bcc is not None else None
            to = self.prep_addresses(self.to.split(';'), **kwargs) if self.to is not None else None
        except LookupError as e:
            logger = logging.getLogger(__name__)
            logger.error("There was an error while parsing the email recipients:\n" + e)
            return
        with open(self.message_html) as html:
            message = Template(html.read())
        with open(self.message_body) as body:
            kwargs.update({
                'body': body.read()
            })
            context = Context(kwargs)
            message = message.render(context)
        EmailMessage(subject='subject', body=message, from_email=settings.AUTOMATED_EMAIL_ADDRESS,
                     to=to, bcc=bcc, cc=cc).send()



# -*- coding: utf-8 -*-
from south.db import db
from south.v2 import SchemaMigration


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Deleting model 'Closure'
        db.delete_table(u'event_procedures_closure')

        # Deleting model 'Email'
        db.delete_table(u'event_procedures_email')

        # Deleting field 'Action.closure'
        db.delete_column(u'event_procedures_action', 'closure_id')

        # Adding field 'Action.content_type'
        db.add_column(u'event_procedures_action', 'content_type',
                      self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType'], null=True),
                      keep_default=False)

        # Adding field 'Action.object_id'
        db.add_column(u'event_procedures_action', 'object_id',
                      self.gf('django.db.models.fields.PositiveIntegerField')(null=True),
                      keep_default=False)


    def backwards(self, orm):
        # Adding model 'Closure'
        db.create_table(u'event_procedures_closure', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
        ))
        db.send_create_signal(u'event_procedures', ['Closure'])

        # Adding model 'Email'
        db.create_table(u'event_procedures_email', (
            (u'closure_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['event_procedures.Closure'], unique=True, primary_key=True)),
        ))
        db.send_create_signal(u'event_procedures', ['Email'])

        # Adding field 'Action.closure'
        db.add_column(u'event_procedures_action', 'closure',
                      self.gf('django.db.models.fields.related.ForeignKey')(to=orm['event_procedures.Closure'], null=True),
                      keep_default=False)

        # Deleting field 'Action.content_type'
        db.delete_column(u'event_procedures_action', 'content_type_id')

        # Deleting field 'Action.object_id'
        db.delete_column(u'event_procedures_action', 'object_id')


    models = {
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'event_procedures.action': {
            'Meta': {'object_name': 'Action'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']", 'null': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['event_procedures.Action']", 'null': 'True'})
        },
        u'event_procedures.event': {
            'Meta': {'object_name': 'Event'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'event_procedures.procedure': {
            'Meta': {'object_name': 'Procedure'},
            'event': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['event_procedures.Event']", 'null': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'root': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['event_procedures.Action']", 'null': 'True'})
        },
    }

    complete_apps = ['event_procedures']



from setuptools import setup


setup(
    name="django-event-procedures",
    version="0.1.3",
    author="Bryan Clement",
    author_email="bclement01@gmail.com",
    description="An event driven code execution system backed by Django",
    long_description=open("README.md", 'rb').read().decode('utf-8'),
    license="GNU",
    url="http://www.github.com/MindClickGlobal/django-event-procedures",
    include_package_data=True,
    packages=['event_procedures'],
    install_requires=[
        "django >= 1.5.0",
        "django-genericadmin >= 0.6.1"
    ],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Internet :: WWW/HTTP :: WSGI",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ])
